// 2D reorderable matrix MAT259 Project 3, Winter 2015, Matthias Wagner
// initialize tables
int [][] dataMatrix = null; 
int [][] rowWeightedMatrix = null; 
int [][] columnWeightedMatrix = null; 
int [][] maxCellMatrix = null;
int [][] cellInRow = null;
int [][] cellInRowCopy = null;

Table myTable; 
int numRows; 
int numColumns;
int maxCount; 
// variables for row-weighted
int [] sumRows = null; 
int [] sumRowsCopy = null;
int [] rowIndexTracker = null;
int [] orderedRows = null; 
// variables for column-weighted
int [] sumColumns = null; 
int [] sumColumnsCopy = null;
int [] columnIndexTracker = null;
int [] orderedColumns = null; 
int [] maxInRow = null; 
int [][] maxColumnIndexTracker = null;
// display options
String [] orderedCategories = null; 
int colorPicker = 20; 
int cellWidth = 10; 
int cellHeight = 50; 
int xMargin, yMargin; 
int infoX, infoY;
int hoverX, hoverY; 
// labels
PFont font; 
String[] categories = { 
  "Adult VHS", "Adult DVD", "Youth VHS", "Youth DVD", "CD-ROM", 
  "Audio CD", "Audio Cassette", "Diskette", "Other"
};
String[] years = { 
  "2005", "2006", "2007", "2008", "2009", "2010", "2011", "2012"
};
// variables for hover
int currentValue; 
String currentCategory; 
int currentMonth;
int currentYear; 

boolean initialFlag = true; 
boolean rowFlag = false; 
boolean columnFlag = false; 
boolean maxCellFlag = false; 

void setup() {
  size(1280, 800); 
  smooth(); 

  font = createFont("Futura", 10);

  myTable = loadTable("GodB.csv", "header");
  numRows = myTable.getColumnCount(); 
  numColumns = myTable.getRowCount(); 

  dataMatrix = new int[numColumns][numRows];
  cellInRowCopy = new int[numRows][numColumns]; 
  rowWeightedMatrix = new int[numColumns][numRows];
  columnWeightedMatrix = new int[numColumns][numRows]; 

  maxCount = dataMatrix[0][0]; 
  sumRows = new int[numRows];
  sumRowsCopy = new int[numRows];
  rowIndexTracker = new int[numRows];
  orderedRows = new int[numRows];
  orderedCategories = new String[numRows];

  sumColumns = new int[numColumns];
  sumColumnsCopy = new int[numColumns];
  columnIndexTracker = new int[numColumns];
  orderedColumns = new int[numColumns];
  maxColumnIndexTracker = new int[numRows][numColumns];

  for ( int i=0; i<numColumns; i++) {
    for ( int j=0; j<numRows; j++) {
      dataMatrix[i][j] = myTable.getInt(i, j);

      if ( dataMatrix[i][j] > maxCount) {
        maxCount = dataMatrix[i][j];
      }
    }
  }
  println("  Rows: " + numRows + "\t\tColumns: " + numColumns + "\tMax Value: " + maxCount );

  // calculate rows
  for ( int i=0; i<numColumns; i++ ) {
    for ( int j=0; j<numRows; j++ ) {
      sumRows[j] += dataMatrix[i][j];
    }
  }
  for ( int i=0; i<numRows; i++ ) {
    sumRowsCopy[i] = sumRows[i];
    orderedCategories[i] = categories[i];
    rowIndexTracker[i] = i; 
    //println("row # " + i + " sum is: " + sumRows[i]);
  }

  for ( int i=0; i<numRows; i++) {
    for ( int j=i+1; j<numRows; j++) {
      if ( sumRowsCopy[i] < sumRowsCopy[j] ) {
        int tempSumRows = sumRowsCopy[i]; 
        sumRowsCopy[i] = sumRowsCopy[j]; 
        sumRowsCopy[j] = tempSumRows; 

        int tempRowIndex = rowIndexTracker[i]; 
        rowIndexTracker[i] = rowIndexTracker[j]; 
        rowIndexTracker[j] = tempRowIndex; 

        String tempCategory = orderedCategories[i]; 
        orderedCategories[i] = orderedCategories[j]; 
        orderedCategories[j] = tempCategory;
      }
    }
  }
  for ( int i=0; i<numColumns; i++) {
    for ( int j=0; j<numRows; j++) {
      int rowPointer = rowIndexTracker[j]; 
      rowWeightedMatrix[i][j] = dataMatrix[i][int(rowPointer)];
    }
  }

  // calculate columns
  for ( int i=0; i<numRows; i++ ) {
    for ( int j=0; j<numColumns; j++ ) {
      sumColumns[j] += dataMatrix[j][i];
      maxColumnIndexTracker[i][j] = (i+1)*(j+1);
    }
  }
  for ( int i=0; i<numColumns; i++ ) {
    sumColumnsCopy[i] = sumColumns[i];
    columnIndexTracker[i] = i; 
    //println("column # " + i + " sum is: " + sumColumns[i]);
  }
  for ( int i=0; i<numColumns; i++) {
    for ( int j=i+1; j<numColumns; j++) {
      if ( sumColumnsCopy[i] < sumColumnsCopy[j] ) {
        int tempSumColumns = sumColumnsCopy[i]; 
        sumColumnsCopy[i] = sumColumnsCopy[j]; 
        sumColumnsCopy[j] = tempSumColumns; 

        int tempColumnIndex = columnIndexTracker[i]; 
        columnIndexTracker[i] = columnIndexTracker[j]; 
        columnIndexTracker[j] = tempColumnIndex;
      }
    }
  }


  for ( int i=0; i<numColumns; i++) {
    for ( int j=0; j<numRows; j++) {
      int columnPointer = columnIndexTracker[i]; 
      int maxColumnPointer = maxColumnIndexTracker[j][i];
      columnWeightedMatrix[i][j] = dataMatrix[int(columnPointer)][j];
    }
  }
}


void draw() {
  colorMode(RGB);
  background(120, 120, 110);

  xMargin = int((width - (cellWidth * numColumns))/2);
  yMargin = int((height - (cellHeight * numRows))/2);
  strokeWeight(1);

  if ( initialFlag ) {
    for ( int i=0; i<numColumns-1; i++) { 
      for ( int j=0; j<numRows; j++) { 
        rectMode(CORNER);
        stroke(60);
        colorMode(HSB);
        fill(colorPicker, 128, pow((255*dataMatrix[i][j]/maxCount), 0.36) * 36);
        rect(xMargin + i*cellWidth +cellWidth/2, yMargin + j*cellHeight +cellHeight/2, cellWidth, cellHeight);
        if ( mouseX >= xMargin+(i-1)*cellWidth && mouseX >= xMargin+(i+1)*cellWidth 
          && mouseY >= yMargin + j*cellHeight && mouseY <= yMargin + (j+1)*cellHeight) {
          currentValue = dataMatrix[i][j];
          currentCategory = categories[j]; 
          currentMonth = (i%12)+1;
          currentYear = 2005 + i/12;
        }
      }
    }
    textAlign(CENTER, CENTER);
    textSize(13);
    colorMode(RGB);
    fill(255, 255, 220);
    for ( int i=0; i<numRows; i++) {
      text( categories[i], xMargin*0.6, yMargin + ((i+1)*cellHeight));
    }
  }


  if ( rowFlag ) {
    for ( int i=0; i<numColumns-1; i++) { 
      for ( int j=0; j<numRows; j++) { 
        rectMode(CORNER);
        stroke(60);
        colorMode(HSB);
        fill(colorPicker, 128, pow((255*rowWeightedMatrix[i][j]/maxCount), 0.36) * 36);
        rect(xMargin + i*cellWidth +cellWidth/2, yMargin + j*cellHeight +cellHeight/2, cellWidth, cellHeight);
        if ( mouseX >= xMargin+(i-1)*cellWidth && mouseX >= xMargin+(i+1)*cellWidth 
          && mouseY >= yMargin + j*cellHeight && mouseY <= yMargin + (j+1)*cellHeight) {
          currentValue = rowWeightedMatrix[i][j];
          currentCategory = orderedCategories[j]; 
          currentMonth = (i%12)+1;
          currentYear = 2005 + i/12;
        }
      }
    }
    textAlign(CENTER, CENTER);
    textSize(13);
    colorMode(RGB);
    fill(255, 255, 220);
    for ( int i=0; i<numRows; i++) {
      text( orderedCategories[i], xMargin*0.6, yMargin + ((i+1)*cellHeight));
    }
  }

  if ( columnFlag ) {
    for ( int i=0; i<numColumns-1; i++) { 
      for ( int j=0; j<numRows; j++) { 
        rectMode(CORNER);
        stroke(60);
        colorMode(HSB);
        fill(colorPicker, 128, pow((255*columnWeightedMatrix[i][j]/maxCount), 0.36) * 36);
        rect(xMargin + i*cellWidth +cellWidth/2, yMargin + j*cellHeight +cellHeight/2, cellWidth, cellHeight);
        if ( mouseX >= xMargin+(i-1)*cellWidth && mouseX >= xMargin+(i+1)*cellWidth 
          && mouseY >= yMargin + j*cellHeight && mouseY <= yMargin + (j+1)*cellHeight) {
          currentValue = columnWeightedMatrix[i][j];
          currentCategory = categories[j]; 
          currentMonth = 0;
          currentYear = 0;
        }
      }
    }
    textAlign(CENTER, CENTER);
    textSize(13);
    colorMode(RGB);
    fill(255, 255, 220);
    for ( int i=0; i<numRows; i++) {
      text( categories[i], xMargin*0.6, yMargin + ((i+1)*cellHeight));
    }
  }

  int colorBarxMargin = xMargin+cellWidth*48; 
  int colorBaryMargin = int(height - yMargin/2); 

  textFont(font, 13); 
  textAlign(CENTER, CENTER);
  for ( int i=0; i<cellWidth*48; i++) { 
    colorMode(HSB);
    stroke(colorPicker, 128, map(i, 0, 255, 0, cellWidth*18)); 
    if ( i%60==0 ) {
      stroke(60);
    }
    line(colorBarxMargin + i + cellWidth/2, colorBaryMargin, colorBarxMargin + i + cellWidth/2, colorBaryMargin+20);
    colorMode(RGB);
    fill(255, 255, 220);
    if ( i<=8 ) {
      text(i*200, colorBarxMargin+cellWidth/2 + i*colorBarxMargin/10.6, colorBaryMargin+30);
    }
  }


  textAlign(CENTER, CENTER);
  textSize(18);
  text("God's Preferred Electronic Medium", width/2, yMargin/1.7);

  textSize(13);
  if ( !columnFlag && !maxCellFlag ) {
    for ( int i=0; i<numColumns/12; i++ ) {
      text(years[i], xMargin + cellWidth*6.5 + i*12 * cellWidth, yMargin+((numRows+0.5)*cellHeight)+25);
    }
    for ( int i=1; i<=96; i+=12) {
      stroke(128, 128, 110);
      strokeWeight(1.5);
      line(xMargin + (i-1)*cellWidth + cellWidth/2, yMargin+cellHeight/2, xMargin+(i-1)*cellWidth + cellWidth/2, yMargin+cellHeight/2 + cellHeight*numRows);
    }
  }


  infoX = int(width/1.4);
  infoY = int(yMargin/1.75);

  textAlign(LEFT, CENTER); 
  textSize(14);
  text("number of check outs: " + "\ncategory: " + "\nmonth: " + "\nyear: ", infoX, infoY);
  if ( mouseX > xMargin+(cellWidth) && mouseX < xMargin+((numColumns-1)*cellWidth + cellWidth/2) 
    && mouseY > yMargin + cellHeight/2 && mouseY < yMargin + (numRows*cellHeight) + cellHeight/2) {
    colorMode(RGB);
    stroke(255, 255, 220);
    noFill();
    rectMode(CORNER);
    hoverX = mouseX - (mouseX%cellWidth); 
    hoverY = mouseY - (mouseY%cellHeight);
    rect(hoverX, hoverY, cellWidth, cellHeight);


    textAlign(RIGHT, CENTER); 
    text(currentValue + "\n" + currentCategory + "\n" + currentMonth + "\n" +  currentYear, infoX+200, infoY);
  }
  textAlign(LEFT, CENTER);
  text("'u' for unsorted table" + "\n" + "'r' for row-sorted table" + "\n" + "'c' for column-sorted table" + "\n" + "'x' to randomize color", xMargin, infoY); 
}

void keyPressed() {
  if ( key == 'r' ) {
    initialFlag = false;  
    columnFlag = false;
    maxCellFlag = false; 
    rowFlag = true;
  }
  if ( key == 'c' ) {
    initialFlag = false;  
    rowFlag = false;
    maxCellFlag = false; 
    columnFlag = true;
  }
  if ( key == 'u' ) {
    rowFlag = false; 
    columnFlag = false; 
    maxCellFlag = false;    
    initialFlag = true;
  }
  if ( key == 'x' ) 
    colorPicker = int(random(0, 255));

  if ( key == 'z' ) 
    colorPicker = 20;

//     if( key == ' ' )
//      saveFrame( "images/screenShot####.jpg" ); 
//      println(" screenshot taken ");
}

